@@ -8,6 +8,7 @@ import logging
 import socket
 import ssl
 
+import pika.compat
 from pika import connection
 from pika import exceptions
 
@@ -16,6 +17,14 @@ try:
 except AttributeError:
     SOL_TCP = 6
 
+
+if pika.compat.PY2:
+    _SOCKET_ERROR = socket.error
+else:
+    # socket.error was deprecated and replaced by OSError in python 3.3
+    _SOCKET_ERROR = OSError
+
+
 LOGGER = logging.getLogger(__name__)
 
 
@@ -27,6 +36,7 @@ class BaseConnection(connection.Connection):
     WRITE = 0x0004
     ERROR = 0x0008
 
+    ERRORS_TO_ABORT = [errno.EBADF, errno.ECONNABORTED, errno.EPIPE]
     ERRORS_TO_IGNORE = [errno.EWOULDBLOCK, errno.EAGAIN, errno.EINTR]
     DO_HANDSHAKE = True
     WARN_ABOUT_IOLOOP = False
@@ -61,13 +71,11 @@ class BaseConnection(connection.Connection):
             raise RuntimeError("SSL specified but it is not available")
         self.base_events = self.READ | self.ERROR
         self.event_state = self.base_events
-        self.fd = None
         self.ioloop = ioloop
         self.socket = None
         self.stop_ioloop_on_close = stop_ioloop_on_close
         self.write_buffer = None
-        super(BaseConnection, self).__init__(parameters,
-                                             on_open_callback,
+        super(BaseConnection, self).__init__(parameters, on_open_callback,
                                              on_open_error_callback,
                                              on_close_callback)
 
@@ -105,38 +113,50 @@ class BaseConnection(connection.Connection):
         self.ioloop.remove_timeout(timeout_id)
 
     def _adapter_connect(self):
-        """Connect to the RabbitMQ broker, returning True if connected
+        """Connect to the RabbitMQ broker, returning True if connected.
 
-        :rtype: bool
+        :returns: error string or exception instance on error; None on success
 
         """
         # Get the addresses for the socket, supporting IPv4 & IPv6
-        try:
-            addresses = socket.getaddrinfo(self.params.host, self.params.port)
-        except socket.error as error:
-            LOGGER.critical('Could not get addresses to use: %s (%s)',
-                            error, self.params.host)
-            return error
+        while True:
+            try:
+                addresses = socket.getaddrinfo(self.params.host, self.params.port,
+                                               0, socket.SOCK_STREAM,
+                                               socket.IPPROTO_TCP)
+                break
+            except _SOCKET_ERROR as error:
+                if error.errno == errno.EINTR:
+                    continue
+
+                LOGGER.critical('Could not get addresses to use: %s (%s)', error,
+                                self.params.host)
+                return error
 
         # If the socket is created and connected, continue on
         error = "No socket addresses available"
         for sock_addr in addresses:
             error = self._create_and_connect_to_socket(sock_addr)
             if not error:
+                # Make the socket non-blocking after the connect
+                self.socket.setblocking(0)
                 return None
+            self._cleanup_socket()
+
         # Failed to connect
         return error
 
     def _adapter_disconnect(self):
         """Invoked if the connection is being told to disconnect"""
-        if hasattr(self, 'heartbeat') and self.heartbeat is not None:
-            self.heartbeat.stop()
-        if self.socket:
-            self.socket.close()
-        self.socket = None
-        self._check_state_on_disconnect()
-        self._handle_ioloop_stop()
-        self._init_connection_state()
+        try:
+            self._remove_heartbeat()
+            self._cleanup_socket()
+            self._check_state_on_disconnect()
+        finally:
+            # Ensure proper cleanup since _check_state_on_disconnect may raise
+            # an exception
+            self._handle_ioloop_stop()
+            self._init_connection_state()
 
     def _check_state_on_disconnect(self):
         """Checks to see if we were in opening a connection with RabbitMQ when
@@ -158,12 +178,25 @@ class BaseConnection(connection.Connection):
             raise exceptions.ProbableAccessDeniedError
         elif self.is_open:
             LOGGER.warning("Socket closed when connection was open")
-        elif not self.is_closed:
+        elif not self.is_closed and not self.is_closing:
             LOGGER.warning('Unknown state on disconnect: %i',
                            self.connection_state)
 
+    def _cleanup_socket(self):
+        """Close the socket cleanly"""
+        if self.socket:
+            try:
+                self.socket.shutdown(socket.SHUT_RDWR)
+            except _SOCKET_ERROR:
+                pass
+            self.socket.close()
+            self.socket = None
+
     def _create_and_connect_to_socket(self, sock_addr_tuple):
-        """Create socket and connect to it, using SSL if enabled."""
+        """Create socket and connect to it, using SSL if enabled.
+
+        :returns: error string on failure; None on success
+        """
         self.socket = socket.socket(sock_addr_tuple[0], socket.SOCK_STREAM, 0)
         self.socket.setsockopt(SOL_TCP, socket.TCP_NODELAY, 1)
         self.socket.settimeout(self.params.socket_timeout)
@@ -175,20 +208,22 @@ class BaseConnection(connection.Connection):
         else:
             ssl_text = ""
 
-        LOGGER.info('Connecting to %s:%s%s',
-                    sock_addr_tuple[4][0], sock_addr_tuple[4][1], ssl_text)
+        LOGGER.info('Connecting to %s:%s%s', sock_addr_tuple[4][0],
+                    sock_addr_tuple[4][1], ssl_text)
 
         # Connect to the socket
         try:
             self.socket.connect(sock_addr_tuple[4])
         except socket.timeout:
             error = 'Connection to %s:%s failed: timeout' % (
-                sock_addr_tuple[4][0], sock_addr_tuple[4][1])
+                sock_addr_tuple[4][0], sock_addr_tuple[4][1]
+            )
             LOGGER.error(error)
             return error
-        except socket.error as error:
-            error = 'Connection to %s:%s failed: %s' % (
-                sock_addr_tuple[4][0], sock_addr_tuple[4][1], error)
+        except _SOCKET_ERROR as error:
+            error = 'Connection to %s:%s failed: %s' % (sock_addr_tuple[4][0],
+                                                        sock_addr_tuple[4][1],
+                                                        error)
             LOGGER.warning(error)
             return error
 
@@ -198,7 +233,8 @@ class BaseConnection(connection.Connection):
                 self._do_ssl_handshake()
             except ssl.SSLError as error:
                 error = 'SSL connection to %s:%s failed: %s' % (
-                    sock_addr_tuple[4][0], sock_addr_tuple[4][1], error)
+                    sock_addr_tuple[4][0], sock_addr_tuple[4][1], error
+                )
                 LOGGER.error(error)
                 return error
         # Made it this far
@@ -223,7 +259,8 @@ class BaseConnection(connection.Connection):
                     raise
                 self._manage_event_state()
 
-    def _get_error_code(self, error_value):
+    @staticmethod
+    def _get_error_code(error_value):
         """Get the error code from the error_value accounting for Python
         version differences.
 
@@ -239,7 +276,10 @@ class BaseConnection(connection.Connection):
         return None
 
     def _flush_outbound(self):
-        """Call the state manager who will figure out that we need to write."""
+        """write early, if the socket will take the data why not get it out
+        there asap.
+        """
+        self._handle_write()
         self._manage_event_state()
 
     def _handle_disconnect(self):
@@ -277,9 +317,9 @@ class BaseConnection(connection.Connection):
             LOGGER.debug("Ignoring %s", error_code)
             return
 
-        # Socket is closed, so lets just go to our handle_close method
-        elif error_code in (errno.EBADF, errno.ECONNABORTED):
-            LOGGER.error("Socket is closed")
+        # Socket is no longer connected, abort
+        elif error_code in self.ERRORS_TO_ABORT:
+            LOGGER.error("Fatal Socket Error: %r", error_value)
 
         elif self.params.ssl and isinstance(error_value, ssl.SSLError):
 
@@ -288,19 +328,21 @@ class BaseConnection(connection.Connection):
             elif error_value.args[0] == ssl.SSL_ERROR_WANT_WRITE:
                 self.event_state = self.WRITE
             else:
-                LOGGER.error("SSL Socket error on fd %d: %r",
-                             self.socket.fileno(), error_value)
-        elif error_code == errno.EPIPE:
-            # Broken pipe, happens when connection reset
-            LOGGER.error("Socket connection was broken")
+                LOGGER.error("SSL Socket error: %r", error_value)
+
         else:
             # Haven't run into this one yet, log it.
-            LOGGER.error("Socket Error on fd %d: %s",
-                         self.socket.fileno(), error_code)
+            LOGGER.error("Socket Error: %s", error_code)
 
         # Disconnect from our IOLoop and let Connection know what's up
         self._handle_disconnect()
 
+    def _handle_timeout(self):
+        """Handle a socket timeout in read or write.
+        We don't do anything in the non-blocking handlers because we
+        only have the socket in a blocking state during connect."""
+        pass
+
     def _handle_events(self, fd, events, error=None, write_only=False):
         """Handle IO/Event loop events, processing them.
 
@@ -310,36 +352,58 @@ class BaseConnection(connection.Connection):
         :param bool write_only: Only handle write events
 
         """
-        if not fd:
-            LOGGER.error('Received events on closed socket: %d', fd)
+        if not self.socket:
+            LOGGER.error('Received events on closed socket: %r', fd)
             return
 
-        if events & self.WRITE:
+        if self.socket and (events & self.WRITE):
             self._handle_write()
             self._manage_event_state()
 
-        if not write_only and (events & self.READ):
+        if self.socket and not write_only and (events & self.READ):
             self._handle_read()
 
-        if write_only and (events & self.READ) and (events & self.ERROR):
+        if (self.socket and write_only and (events & self.READ) and
+            (events & self.ERROR)):
             LOGGER.error('BAD libc:  Write-Only but Read+Error. '
                          'Assume socket disconnected.')
             self._handle_disconnect()
 
-        if events & self.ERROR:
+        if self.socket and (events & self.ERROR):
             LOGGER.error('Error event %r, %r', events, error)
             self._handle_error(error)
 
     def _handle_read(self):
         """Read from the socket and call our on_data_available with the data."""
         try:
-            if self.params.ssl:
-                data = self.socket.read(self._buffer_size)
-            else:
-                data = self.socket.recv(self._buffer_size)
+            while True:
+                try:
+                    if self.params.ssl:
+                        data = self.socket.read(self._buffer_size)
+                    else:
+                        data = self.socket.recv(self._buffer_size)
+
+                    break
+                except _SOCKET_ERROR as error:
+                    if error.errno == errno.EINTR:
+                        continue
+                    else:
+                        raise
+
         except socket.timeout:
-            raise
-        except socket.error as error:
+            self._handle_timeout()
+            return 0
+
+        except ssl.SSLError as error:
+            if error.args[0] == ssl.SSL_ERROR_WANT_READ:
+                # ssl wants more data but there is nothing currently
+                # available in the socket, wait for it to become readable.
+                return 0
+            return self._handle_error(error)
+
+        except _SOCKET_ERROR as error:
+            if error.errno in (errno.EAGAIN, errno.EWOULDBLOCK):
+                return 0
             return self._handle_error(error)
 
         # Empty data, should disconnect
@@ -352,19 +416,44 @@ class BaseConnection(connection.Connection):
         return len(data)
 
     def _handle_write(self):
-        """Handle any outbound buffer writes that need to take place."""
+        """Try and write as much as we can, if we get blocked requeue
+        what's left"""
         bytes_written = 0
-        if self.outbound_buffer:
-            frame = self.outbound_buffer.popleft()
-            try:
-                self.socket.sendall(frame)
-                bytes_written = len(frame)
-            except socket.timeout:
-                raise
-            except socket.error as error:
+        try:
+            while self.outbound_buffer:
+                frame = self.outbound_buffer.popleft()
+                while True:
+                    try:
+                        bw = self.socket.send(frame)
+                        break
+                    except _SOCKET_ERROR as error:
+                        if error.errno == errno.EINTR:
+                            continue
+                        else:
+                            raise
+
+                bytes_written += bw
+                if bw < len(frame):
+                    LOGGER.debug("Partial write, requeing remaining data")
+                    self.outbound_buffer.appendleft(frame[bw:])
+                    break
+
+        except socket.timeout:
+            # Will only come here if the socket is blocking
+            LOGGER.debug("socket timeout, requeuing frame")
+            self.outbound_buffer.appendleft(frame)
+            self._handle_timeout()
+
+        except _SOCKET_ERROR as error:
+            if error.errno in (errno.EAGAIN, errno.EWOULDBLOCK):
+                LOGGER.debug("Would block, requeuing frame")
+                self.outbound_buffer.appendleft(frame)
+            else:
                 return self._handle_error(error)
+
         return bytes_written
 
+
     def _init_connection_state(self):
         """Initialize or reset all of our internal state variables for a given
         connection. If we disconnect and reconnect, all of our state needs to
@@ -372,7 +461,6 @@ class BaseConnection(connection.Connection):
 
         """
         super(BaseConnection, self)._init_connection_state()
-        self.fd = None
         self.base_events = self.READ | self.ERROR
         self.event_state = self.base_events
         self.socket = None
